from jsonschema import validate
import functools
from flask import request


#schema that defines valid json
schema = {
     "type" : "object",
     "properties" : {
         "latitud" : {"type" : "number",
                      "minimum" : -90,
                      "maximum": 90},
        "longitud" : {"type" : "number",
                       "minimum" : -180,
                       "maximum" : 180},
     },
     "required" : ["latitud", "longitud"],
     "propertyNames": {"enum": ["latitud", "longitud"]}   
 }


def validate_schema(json, schema):
    ''' validates json from request flask object according to a schema
        INPUT -  json: json to validate
                schema: jsonschema defininig valid json structure.
                See: https://json-schema.org/
    '''
    try:
        validate(instance=json, schema=schema)
        errors = None
    except Exception as error:
        errors = str(error)
    return errors

def valid_json(schema):
    '''Checks if json is valid according to a schema.  If the json
    is not valid returns a response with the errors.
    '''
    def decorator(func):
        @functools.wraps(func)
        def wrapper_valid_json(*args, **kwargs):
            #validate json
            json = request.get_json(silent=True)
            errors = validate_schema(json, schema)
            #if json is invalid return response with errors
            if errors:
                response = 'Invalid json. Errors: {}'.format(errors)
                return response
            #else run function
            else:
                return func(*args, **kwargs)
        return wrapper_valid_json
    return decorator
